@@ -8,6 +8,7 @@ Inherit stock for endicia API
 from decimal import Decimal
 import base64
 import math
+import logging
 
 from endicia import ShippingLabelAPI, LabelRequest, RefundRequestAPI, \
     BuyingPostageAPI, Element, CalculatingPostageAPI
@@ -26,7 +27,7 @@ from .sale import ENDICIA_PACKAGE_TYPES, MAILPIECE_SHAPES
 
 __metaclass__ = PoolMeta
 __all__ = [
-    'ShipmentOut', 'GenerateEndiciaLabelMessage', 'GenerateEndiciaLabel',
+    'ShipmentOut', 'ShippingEndicia', 'GenerateShippingLabel',
     'EndiciaRefundRequestWizardView', 'EndiciaRefundRequestWizard',
     'BuyPostageWizardView', 'BuyPostageWizard',
 ]
@@ -35,6 +36,8 @@ STATES = {
     'readonly': Eval('state') == 'done',
 }
 
+logger = logging.getLogger(__name__)
+
 
 class ShipmentOut:
     "Shipment Out"
@@ -189,7 +192,6 @@ class ShipmentOut:
 
         user = User(Transaction().user)
         customsitems = []
-        value = 0
 
         for move in self.outgoing_moves:
             if move.quantity <= 0:
@@ -198,10 +200,9 @@ class ShipmentOut:
                 Element('Description', move.product.name[0:50]),
                 Element('Quantity', int(math.ceil(move.quantity))),
                 Element('Weight', int(move.get_weight(self.weight_uom))),
-                Element('Value', float(move.product.list_price)),
+                Element('Value', float(move.product.customs_value_used)),
             ]
             customsitems.append(Element('CustomsItem', new_item))
-            value += float(move.product.list_price) * move.quantity
 
         description = ','.join([
             move.product.name for move in self.outgoing_moves
@@ -213,14 +214,7 @@ class ShipmentOut:
                 Element('ContentsType', self.endicia_package_type)
             ]
         })
-        total_value = sum(map(
-            lambda move: float(move.product.cost_price) * move.quantity,
-            self.outgoing_moves
-        ))
         request.add_data({
-            'ContentsType': self.endicia_package_type,
-            'Value': total_value,
-            'Description': description[:50],
             'CustomsCertify': 'TRUE',   # TODO: Should this be part of config ?
             'CustomsSigner': user.name,
         })
@@ -301,6 +295,16 @@ class ShipmentOut:
 
         self._update_endicia_item_details(shipping_label_request)
 
+        # Logging.
+        logger.debug(
+            'Making Shipping Label Request for'
+            'Shipment ID: {0} and Carrier ID: {1}'
+            .format(self.id, self.carrier.id)
+        )
+        logger.debug('--------SHIPPING LABEL REQUEST--------')
+        logger.debug(str(shipping_label_request.to_xml()))
+        logger.debug('--------END REQUEST--------')
+
         try:
             response = shipping_label_request.send_request()
         except RequestError, error:
@@ -308,6 +312,11 @@ class ShipmentOut:
         else:
             result = objectify_response(response)
 
+            # Logging.
+            logger.debug('--------SHIPPING LABEL RESPONSE--------')
+            logger.debug(str(response))
+            logger.debug('--------END RESPONSE--------')
+
             tracking_number = result.TrackingNumber.pyval
             self.__class__.write([self], {
                 'tracking_number': unicode(result.TrackingNumber.pyval),
@@ -323,7 +332,7 @@ class ShipmentOut:
                     'resource': '%s,%s' % (self.__name__, self.id)
                 }])
 
-            return tracking_number
+            return str(tracking_number)
 
     def _get_ship_from_address(self):
         """
@@ -366,11 +375,26 @@ class ShipmentOut:
             test=endicia_credentials.is_test,
         )
 
+        # Logging.
+        logger.debug(
+            'Making Postage Request for'
+            'Shipment ID: {0} and Carrier ID: {1}'
+            .format(self.id, self.carrier.id)
+        )
+        logger.debug('--------POSTAGE REQUEST--------')
+        logger.debug(str(calculate_postage_request.to_xml()))
+        logger.debug('--------END REQUEST--------')
+
         try:
             response = calculate_postage_request.send_request()
         except RequestError, error:
             self.raise_user_error('error_label', error_args=(error,))
 
+        # Logging.
+        logger.debug('--------POSTAGE RESPONSE--------')
+        logger.debug(str(response))
+        logger.debug('--------END RESPONSE--------')
+
         return Decimal(
             objectify_response(response).PostagePrice.get('TotalAmount')
         )
@@ -382,40 +406,6 @@ class ShipmentOut:
         return self.carrier and self.carrier.carrier_cost_method == 'endicia'
 
 
-class GenerateEndiciaLabelMessage(ModelView):
-    'Generate Endicia Labels Message'
-    __name__ = 'generate.endicia.label.message'
-
-    tracking_number = fields.Char("Tracking number", readonly=True)
-
-
-class GenerateEndiciaLabel(Wizard):
-    'Generate Endicia Labels'
-    __name__ = 'generate.endicia.label'
-
-    start = StateView(
-        'generate.endicia.label.message',
-        'endicia_integration.generate_endicia_label_message_view_form',
-        [
-            Button('Ok', 'end', 'tryton-ok'),
-        ]
-    )
-
-    def default_start(self, data):
-        Shipment = Pool().get('stock.shipment.out')
-
-        try:
-            shipment, = Shipment.browse(Transaction().context['active_ids'])
-        except ValueError:
-            self.raise_user_error(
-                'This wizard can be called for only one shipment at a time'
-            )
-
-        tracking_number = shipment.make_endicia_labels()
-
-        return {'tracking_number': str(tracking_number)}
-
-
 class EndiciaRefundRequestWizardView(ModelView):
     """Endicia Refund Wizard View
     """
@@ -569,3 +559,106 @@ class BuyPostageWizard(Wizard):
         default['response'] = str(result.ErrorMessage) \
             if hasattr(result, 'ErrorMessage') else 'Success'
         return default
+
+
+class ShippingEndicia(ModelView):
+    'Endicia Configuration'
+    __name__ = 'shipping.label.endicia'
+
+    endicia_mailclass = fields.Many2One(
+        'endicia.mailclass', 'MailClass', required=True
+    )
+    endicia_mailpiece_shape = fields.Selection(
+        MAILPIECE_SHAPES, 'Endicia MailPiece Shape'
+    )
+    endicia_shipment_bag = fields.Many2One(
+        'endicia.shipment.bag', 'Endicia Shipment Bag'
+    )
+    endicia_label_subtype = fields.Selection([
+        ('None', 'None'),
+        ('Integrated', 'Integrated')
+    ], 'Label Subtype')
+    endicia_integrated_form_type = fields.Selection([
+        (None, ''),
+        ('Form2976', 'Form2976(Same as CN22)'),
+        ('Form2976A', 'Form2976(Same as CP72)'),
+    ], 'Integrated Form Type')
+    endicia_include_postage = fields.Boolean('Include Postage ?')
+    endicia_package_type = fields.Selection(
+        ENDICIA_PACKAGE_TYPES, 'Package Content Type'
+    )
+    endicia_refunded = fields.Boolean('Refunded ?', readonly=True)
+
+
+class GenerateShippingLabel(Wizard):
+    'Generate Labels'
+    __name__ = 'shipping.label'
+
+    endicia_config = StateView(
+        'shipping.label.endicia',
+        'endicia_integration.shipping_endicia_configuration_view_form',
+        [
+            Button('Back', 'start', 'tryton-go-previous'),
+            Button('Continue', 'generate', 'tryton-go-next'),
+        ]
+    )
+
+    def default_endicia_config(self, data):
+        Config = Pool().get('sale.configuration')
+        config = Config(1)
+        shipment = self.start.shipment
+
+        return {
+            'endicia_mailclass': (
+                shipment.endicia_mailclass and shipment.endicia_mailclass.id
+            ) or (
+                config.endicia_mailclass and config.endicia_mailclass.id
+            ) or None,
+            'endicia_mailpiece_shape': (
+                shipment.endicia_mailpiece_shape or
+                config.endicia_mailpiece_shape
+            ),
+            'endicia_label_subtype': (
+                shipment.endicia_label_subtype or config.endicia_label_subtype
+            ),
+            'endicia_integrated_form_type': (
+                shipment.endicia_integrated_form_type or
+                config.endicia_integrated_form_type
+            ),
+            'endicia_include_postage': (
+                shipment.endicia_include_postage or
+                config.endicia_include_postage
+            ),
+            'endicia_package_type': (
+                shipment.endicia_package_type or config.endicia_package_type
+            )
+        }
+
+    def transition_next(self):
+        state = super(GenerateShippingLabel, self).transition_next()
+
+        if self.start.carrier.carrier_cost_method == 'endicia':
+            return 'endicia_config'
+        return state
+
+    def update_shipment(self):
+        shipment = self.start.shipment
+
+        if self.start.carrier.carrier_cost_method == 'endicia':
+            shipment.endicia_mailclass = self.endicia_config.endicia_mailclass
+            shipment.endicia_mailpiece_shape = \
+                self.endicia_config.endicia_mailpiece_shape
+            shipment.endicia_shipment_bag = \
+                self.endicia_config.endicia_shipment_bag
+            shipment.endicia_label_subtype = \
+                self.endicia_config.endicia_label_subtype
+            shipment.endicia_integrated_form_type = \
+                self.endicia_config.endicia_integrated_form_type
+            shipment.endicia_package_type = \
+                self.endicia_config.endicia_package_type
+            shipment.endicia_include_postage = \
+                self.endicia_config.endicia_include_postage
+            shipment.endicia_shipment_bag = \
+                self.endicia_config.endicia_shipment_bag
+
+        return super(GenerateShippingLabel, self).update_shipment()
